#!/usr/bin/python
#
# genl.py -- generic netlink constants, header, and parsers
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from struct import calcsize, pack

import pymnl
import pymnl.message
from pymnl.attributes import AttrParser

#
# linux/genetlink.h
#
GENL_NAMSIZ = 16     # length of family name

GENL_MIN_ID = pymnl.message.NLMSG_MIN_TYPE
GENL_MAX_ID = 1023

GENL_ADMIN_PERM = 0x01
GENL_CMD_CAP_DO = 0x02
GENL_CMD_CAP_DUMP = 0x04
GENL_CMD_CAP_HASPOL = 0x08

# List of reserved static generic netlink identifiers:
GENL_ID_GENERATE = 0
GENL_ID_CTRL = pymnl.message.NLMSG_MIN_TYPE

# controller commands
CTRL_CMD_UNSPEC = 0
CTRL_CMD_NEWFAMILY = 1
CTRL_CMD_DELFAMILY = 2
CTRL_CMD_GETFAMILY = 3
CTRL_CMD_NEWOPS = 4
CTRL_CMD_DELOPS = 5
CTRL_CMD_GETOPS = 6
CTRL_CMD_NEWMCAST_GRP = 7
CTRL_CMD_DELMCAST_GRP = 8
CTRL_CMD_GETMCAST_GRP = 9     # unused
CTRL_CMD_MAX = 10             # always keep last

# generic netlink controller attribute types
CTRL_ATTR_UNSPEC = 0
CTRL_ATTR_FAMILY_ID = 1
CTRL_ATTR_FAMILY_NAME = 2
CTRL_ATTR_VERSION = 3
CTRL_ATTR_HDRSIZE = 4
CTRL_ATTR_MAXATTR = 5
CTRL_ATTR_OPS = 6
CTRL_ATTR_MCAST_GROUPS = 7
CTRL_ATTR_MAX = 8             # always keep last

CTRL_ATTR_OP_UNSPEC = 0
CTRL_ATTR_OP_ID = 1
CTRL_ATTR_OP_FLAGS = 2
CTRL_ATTR_OP_MAX = 3

CTRL_ATTR_MCAST_GRP_UNSPEC = 0
CTRL_ATTR_MCAST_GRP_NAME = 1
CTRL_ATTR_MCAST_GRP_ID = 2
CTRL_ATTR_MCAST_GRP_MAX = 3


class GenlMessageHeader(object):
    def __init__(self, command=None, version=None):
        """ An extra header for the message.

            command - genl command to issue

            version - genl protocol version

            Implements genlmsghdr in a Pythonesque form.
        """
        self._format = 'BBH'
        self._command = command
        self._version = version
        self._reserved = 0

    def __len__(self):
        """ Calculate and return genlmsghdr length.
        """
        return calcsize(self._format)

    def set_command(self, command):
        """ Set the header command.

            command - genl command to issue
        """
        self._command = command

    def set_version(self, version):
        """ Set the header command.

            version - genl protocol version
        """
        self._version = version

    def get_binary(self):
        """ Return a packed struct suitable for sending through a
            netlink socket.

            Raises an exception if command and version have not been set
            before calling get_binary().
        """
        return pack(self._format,
                        self._command, self._version, self._reserved)


class GenlAttrParser(AttrParser):
    """ Parser for generic netlink attributes.
    """
    def __init__(self):
        # dict to hold attributes without an assigned callback
        self._attributes = { 'unmatched': [] }

        self._cb = {CTRL_ATTR_FAMILY_ID : self.ctrl_attr_family_id,
                    CTRL_ATTR_FAMILY_NAME : self.ctrl_attr_family_name,
                    CTRL_ATTR_VERSION : self.ctrl_attr_version,
                    CTRL_ATTR_HDRSIZE : self.ctrl_attr_hdrsize,
                    CTRL_ATTR_MAXATTR : self.ctrl_attr_maxattr,
                    CTRL_ATTR_OPS : self.ctrl_attr_ops,
                    CTRL_ATTR_MCAST_GROUPS : self.ctrl_attr_mcast_groups}

    def ctrl_attr_family_id(self, attr):
        """ Save family id.

            attr - Attr object
        """
        self._attributes['id'] = attr.get_u16()

    def ctrl_attr_family_name(self, attr):
        """ Save family name.

            attr - Attr object
        """
        self._attributes['name'] = attr.get_str_stripped()

    def ctrl_attr_version(self, attr):
        """ Save version.

            attr - Attr object
        """
        self._attributes['version'] = attr.get_u32()

    def ctrl_attr_hdrsize(self, attr):
        """ Save header size.

            attr - Attr object
        """
        self._attributes['hdrsize'] = attr.get_u32()

    def ctrl_attr_maxattr(self, attr):
        """ Save maximum attribute number.

            attr - Attr object
        """
        self._attributes['maxattr'] = attr.get_u32()

    def ctrl_attr_ops(self, attr):
        """ Print attribute type and parse nested attributes.

            attr - Attr object
        """
        self._attributes['ops'] = {}
        # process a list of nested attributes
        for one_attr in GenlAttrOpParser().parse_nested(attr):
            # get list of nested nested attributes  <-- not a typo
            nested_attrs = GenlAttrOpParser().parse_nested(one_attr)
            # save nested attributes to 'ops' dictionary
            self._attributes['ops'][nested_attrs[0].get_u32()] = \
                                                nested_attrs[1].get_u32()

    def ctrl_attr_mcast_groups(self, attr):
        """ Print attribute type and parse nested attributes.

            attr - Attr object
        """
        self._attributes['groups'] = {}
        # process a list of nested attributes
        for one_attr in GenlAttrGroupParser().parse_nested(attr):
            # get list of nested nested attributes  <-- not a typo
            nested_attrs = GenlAttrGroupParser().parse_nested(one_attr)
            # save nested attributes to 'groups' dictionary
            self._attributes['groups'][nested_attrs[0].get_u32()] = \
                                        nested_attrs[1].get_str_stripped()

    def parse(self, data):
        """ Process the attributes.

            data - object with attributes
        """
        for one_attr in self.parse_string(data.get_binary(), offset=4):
            try:
                self._cb[one_attr.get_type()](one_attr)
            except KeyError:
                self._attributes['unmatched'].append(one_attr)
        return self._attributes


class GenlAttrOpParser(AttrParser):
    """ Parser for generic netlink nested op attributes.
    """
    def __init__(self):
        # list to hold attributes without an assigned callback
        self._attributes = []

        self._cb = {CTRL_ATTR_OP_ID : self.ctrl_attr_op_id}

    def ctrl_attr_op_id(self, attr):
        self._attributes.append(attr.get_u32())


class GenlAttrGroupParser(AttrParser):
    """ Parser for generic netlink nested group attributes.
    """
    def __init__(self):
        # list to hold attributes without an assigned callback
        self._attributes = []

        self._cb = {CTRL_ATTR_MCAST_GRP_ID : self.ctrl_attr_mcast_group_id,
                CTRL_ATTR_MCAST_GRP_NAME : self.ctrl_attr_mcast_group_name}

    def ctrl_attr_mcast_group_id(self, attr):
        self._attributes.append(attr.get_u32())

    def ctrl_attr_mcast_group_name(self, attr):
        self._attributes.append(attr.get_str_stripped())




#!/usr/bin/python
#
# if_link.py -- constants and data structures from linux/if_link.h
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from pymnl.attributes import AttrParser

IFLA_IFNAME = 3
IFLA_MTU = 4

class IFLAttrParser(AttrParser):
    """ Parser for interface attributes.
    """
    def __init__(self):
        # dict to hold attributes without an assigned callback
        self._attributes = { 'unmatched': [] }

        self._cb = {IFLA_MTU : self.ifla_mtu,
                    IFLA_IFNAME : self.ifla_ifname}

    def ifla_mtu(self, attr):
        """ Save MTU.

            attr - Attr object
        """
        self._attributes['mtu'] = attr.get_u32()

    def ifla_ifname(self, attr):
        """ Save interface name.

            attr - Attr object
        """
        self._attributes['ifname'] = attr.get_str_stripped()

    def parse(self, data):
        """ Process the attributes.

            data - object with attributes
        """
        for one_attr in self.parse_string(data.get_binary(), offset=0):
            try:
                self._cb[one_attr.get_type()](one_attr)
            except KeyError:
                self._attributes['unmatched'].append(one_attr)

        return self._attributes




#!/usr/bin/python
#
# nlsocket.py -- interface to netlink socket
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#
# Much of the method docstrings are from libmnl and are
#      Copyright 2008-2010 by Pablo Neira Ayuso <pablo@netfilter.org>
#

from resource import getpagesize
import socket

import pymnl
from pymnl.message import MessageList

NETLINK_ADD_MEMBERSHIP = 1
NETLINK_DROP_MEMBERSHIP = 2
NETLINK_PKTINFO = 3
NETLINK_BROADCAST_ERROR = 4
NETLINK_NO_ENOBUFS = 5

#
# libmnl.h
#

SOL_NETLINK = 270

SOCKET_AUTOPID = 0

SOCKET_BUFFER_SIZE = 8192
if (getpagesize() < 8192):
    SOCKET_BUFFER_SIZE = getpagesize()


class Socket(object):
    def __init__(self, bus):
        """ A netlink socket.

            bus - the netlink socket bus ID
                    (see NETLINK_* constants in linux/netlink.h)

            Raises an exception on error.
        """
        self._bus = bus
        self._groups = 0   # multicast groups mask

        self._socket = socket.socket(socket.AF_NETLINK, socket.SOCK_RAW, bus)

    def get_sock(self):
        """ Get the underlying socket object.

            This is useful if you need to set non-netlink socket options.
        """
        return self._socket

    def get_portid(self):
        """ Obtain netlink PortID from netlink socket.

            This method returns the netlink PortID of this netlink socket.
            It's a common mistake to assume that this PortID equals the
            process ID which is not always true. This is the case if you
            open more than one socket that is binded to the same netlink
            subsystem from the same process.
        """
        return self._socket.getsockname()[0]

    def get_groups(self):
        """ Obtain netlink groups from netlink socket.
        """
        return self._socket.getsockname()[1]

    def bind(self, pid=SOCKET_AUTOPID, groups=0):
        """ Bind netlink socket.

            pid - The port ID you want to use.  You can use
                    SOCKET_AUTOPID (which is 0) for automatic port ID
                    selection.

            groups - the group of message you're interested in

            Raises an exception on error.
        """
        self._socket.bind((pid, groups))

    def send(self, nl_message):
        """ Send a netlink message.

            nl_message - the netlink message to be sent

            Raises an exception on error. Otherwise, it returns the number of
            bytes sent.
        """
        return self._socket.send(nl_message.get_binary())

    def recv(self, bufsize=SOCKET_BUFFER_SIZE, flags=0):
        """ Receive a netlink message.

            bufsize - max data to receive
                        Use SOCKET_BUFFER_SIZE (which is 8KB, see
                        linux/netlink.h for more information). Using this
                        buffer size ensures that your buffer is big enough
                        to store the netlink message without truncating it.

            flags - see socket.recv()

            Raises an exception on error.  Otherwise, it returns a
            MessageList.
        """
        return MessageList(self._socket.recv(bufsize, flags))

    def close(self):
        """ Close the socket.
        """
        self._socket.close()
        self._bus = None

    def setsockopt(self, optname, value):
        """ Set Netlink socket option.

            optname - option to set

            value - value to set for the option

            This method allows you to set some Netlink socket options. As
            of this writing (see linux/netlink.h), the existing options are:

                - NETLINK_ADD_MEMBERSHIP

                - NETLINK_DROP_MEMBERSHIP

                - NETLINK_PKTINFO

                - NETLINK_BROADCAST_ERROR

                - NETLINK_NO_ENOBUFS

            In the early days, Netlink only supported 32 groups expressed
            in a 32-bits mask. However, since 2.6.14, Netlink may have up
            to 2^32 multicast groups but you have to use setsockopt() with
            NETLINK_ADD_MEMBERSHIP to join a given multicast group. This
            method internally calls setsockopt() to join a given netlink
            multicast group. You can still use Socket.bind() and the 32-bit
            mask to join a set of Netlink multicast groups.

            See Python's socket module for more information about value.
        """
        self._socket.setsockopt(SOL_NETLINK, optname, value)

    def getsockopt(self, optname, buflen=None):
        """ Get a Netlink socket option.

            optname - the option to get

            buflen - optional (see Python's socket module)
        """
        self._socket.getsockopt(SOL_NETLINK, optname, buflen)




# pymnl -- a minimalistic pure Python interface for netlink
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

# This module is heavily influenced by the excellent libmnl
# from Pablo Neira Ayuso <pablo@netfilter.org>.  However,
# pymnl does not use libmnl.

def PYMNL_ALIGN(align_size):
    """ Return a function to calculate alignment.
    """
    return lambda len: (((len) + align_size - 1) & ~(align_size - 1))

#
# linux/netlink.h
#

# netlink types
NETLINK_ROUTE = 0             # Routing/device hook
NETLINK_UNUSED = 1            # Unused number
NETLINK_USERSOCK = 2          # Reserved for user mode socket protocols
NETLINK_FIREWALL = 3          # Firewalling hook
NETLINK_INET_DIAG = 4         # INET socket monitoring
NETLINK_NFLOG = 5             # netfilter/iptables ULOG
NETLINK_XFRM = 6              # ipsec
NETLINK_SELINUX = 7           # SELinux event notifications
NETLINK_ISCSI = 8             # Open-iSCSI
NETLINK_AUDIT = 9             # auditing
NETLINK_FIB_LOOKUP = 10
NETLINK_CONNECTOR = 11
NETLINK_NETFILTER = 12        # netfilter subsystem
NETLINK_IP6_FW = 13
NETLINK_DNRTMSG = 14          # DECnet routing messages
NETLINK_KOBJECT_UEVENT = 15   # Kernel messages to userspace
NETLINK_GENERIC = 16
NETLINK_SCSITRANSPORT = 18    # SCSI Transports
NETLINK_ECRYPTFS = 19

MAX_LINKS = 32

NETLINK_ADD_MEMBERSHIP = 1
NETLINK_DROP_MEMBERSHIP = 2
NETLINK_PKTINFO = 3
NETLINK_BROADCAST_ERROR = 4
NETLINK_NO_ENOBUFS = 5

NET_MAJOR = 36          # Major 36 is reserved for networking




#!/usr/bin/python
#
# rtnl-link-add.py -- add a route
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

import ctypes
from random import randint
import socket
from struct import pack, unpack
import sys

import pymnl
from pymnl.attributes import Attr
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

import if_
import if_link
import rtnetlink

if (len(sys.argv) <= 3):
    print("Usage: %s iface destination cidr [gateway]" % sys.argv[0]);
    print("Example: %s eth0 10.0.1.12 32 10.0.1.11" % sys.argv[0]);
    sys.exit()

libc = ctypes.CDLL("libc.so.6")

iface = libc.if_nametoindex(sys.argv[1].encode())
if (iface == 0):
    print("Bad interface name")
    sys.exit()

try:
    dst = unpack("L", socket.inet_pton(socket.AF_INET, sys.argv[2]))[0]
except socket.error:
    print("Bad destination")
    sys.exit()

try:
    mask = int(sys.argv[3])
except:
    print("Bad CIDR")
    sys.exit()

if (len(sys.argv) >= 5):
    try:
        gw = unpack("L", socket.inet_pton(socket.AF_INET, sys.argv[4]))[0]
    except:
        print("Bad gateway")
        sys.exit()

# init and build request message
rtnlmsg = Message()
rtnlmsg.set_type(rtnetlink.RTM_NEWROUTE)
rtnlmsg.set_flags(pymnl.message.NLM_F_REQUEST | pymnl.message.NLM_F_CREATE)
rtnlmsg.set_seq(randint(0, pow(2, 31)))

# build ifm header and add it to the message
rtm = rtnetlink.RtMessage()
rtm._family = socket.AF_INET
rtm._dst_len = mask
rtm._src_len = 0
rtm._tos = 0
rtm._protocol = rtnetlink.RTPROT_BOOT
rtm._table = rtnetlink.RT_TABLE_MAIN
rtm._type = rtnetlink.RTN_UNICAST
# Is there any gateway?
if (len(sys.argv) == 4):
    rtm._scope = rtnetlink.RT_SCOPE_LINK
else:
    rtm._scope = rtnetlink.RT_SCOPE_UNIVERSE
rtm._flags = 0

rtnlmsg.put_extra_header(rtm)

payload = rtnlmsg.get_payload()
payload.add_attr(Attr.new_u32(rtnetlink.RTA_DST, dst))
payload.add_attr(Attr.new_u32(rtnetlink.RTA_OIF, iface))
if (len(sys.argv) >= 5):
    payload.add_attr(Attr.new_u32(rtnetlink.RTA_GATEWAY, gw))

# init and bind netlink socket
sock = Socket(pymnl.NETLINK_ROUTE)
sock.bind(pymnl.nlsocket.SOCKET_AUTOPID, 0)

# send message through socket
sock.send(rtnlmsg)

# ignore any return messages

sock.close()




#!/usr/bin/python
#
# rtnl-route-dump.py -- dump kernel routing info from rtnetlink
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

from random import randint
import socket
import struct
import sys

import pymnl
from pymnl.attributes import Attr, AttrParser
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

import if_
import if_link
import rtnetlink

class RouteParser(AttrParser):
    """ Parser for routing info.
    """
    def __init__(self):
        # dict to hold attributes without an assigned callback
        self._attributes = { 'unmatched': [] }

        self._cb = {rtnetlink.RTA_TABLE : self.rta_table,
                    rtnetlink.RTA_DST : self.rta_dst,
                    rtnetlink.RTA_SRC : self.rta_src,
                    rtnetlink.RTA_OIF : self.rta_oif,
                    rtnetlink.RTA_FLOW : self.rta_flow,
                    rtnetlink.RTA_PREFSRC : self.rta_prefsrc,
                    rtnetlink.RTA_GATEWAY : self.rta_gateway,
                    rtnetlink.RTA_METRICS : self.rta_metrics}

    def rta_table(self, attr):
        self._attributes['table'] = attr.get_u32()

    def rta_dst(self, attr):
        dst = struct.pack("L", attr.get_u32())
        self._attributes['dst'] = socket.inet_ntoa(dst)

    def rta_src(self, attr):
        src = struct.pack("L", attr.get_u32())
        self._attributes['src'] = socket.inet_ntoa(src)

    def rta_oif(self, attr):
        self._attributes['oif'] = attr.get_u32()

    def rta_flow(self, attr):
        self._attributes['flow'] = attr.get_u32()

    def rta_prefsrc(self, attr):
        prefsrc = struct.pack("L", attr.get_u32())
        self._attributes['prefsrc'] = socket.inet_ntoa(prefsrc)

    def rta_gateway(self, attr):
        gateway = struct.pack("L", attr.get_u32())
        self._attributes['gw'] = socket.inet_ntoa(gateway)

    def rta_metrics(self, attr):
        """ Parse nested attributes.

            attr - Attr object
        """
        self._attributes['metrics'] = {}
        # process a list of nested attributes
        for one_attr in RouteMetricsParser().parse_nested(attr):
            # get list of nested nested attributes  <-- not a typo
            nested_attrs = RouteMetricsParser().parse_nested(one_attr)
            # save nested attributes to 'ops' dictionary
            self._attributes['metrics'][nested_attrs[0].get_u32()] = \
                                                nested_attrs[1].get_u32()

    def parse(self, data, offset=0):
        """ Process the attributes.

            data - object with attributes
        """
        for one_attr in self.parse_string(data.get_binary(), offset):
            try:
                self._cb[one_attr.get_type()](one_attr)
            except KeyError:
                self._attributes['unmatched'].append(one_attr)

        return self._attributes


class RouteMetricsParser(AttrParser):
    """ Parser for route metrics.
    """
    def __init__(self):
        # list to hold attributes without an assigned callback
        self._attributes = []

        self._cb = {RTAX_MAX : self.rtax_max}

    def rtax_max(self, attr):
        self._attributes.append(attr.get_u32())



# init and build request message
rtnlmsg = Message()
rtnlmsg.set_type(rtnetlink.RTM_GETROUTE)
rtnlmsg.set_flags(pymnl.message.NLM_F_REQUEST | pymnl.message.NLM_F_DUMP)

sequence = randint(0, pow(2, 31))
rtnlmsg.set_seq(sequence)

# build rtgenmsg header and add it to the message
rtmsg_header = rtnetlink.RtGenMessageHeader()
rtmsg_header._family = socket.AF_INET
rtnlmsg.put_extra_header(rtmsg_header)

# init and bind netlink socket
sock = Socket(pymnl.NETLINK_ROUTE)
sock.bind(pymnl.nlsocket.SOCKET_AUTOPID, 0)

# send message through socket
sock.send(rtnlmsg)

# read returned message from netlink
return_msg = sock.recv(flags=socket.MSG_DONTWAIT)

sock.close()

# process the messages returned
for msg in return_msg:
    if (msg.get_errno()):
        # tell the user what error occurred
        print("error:", msg.get_errstr())
    else:
        rtm = rtnetlink.RtMessage(packed_data=msg.get_payload())
        line = ""
        # protocol family = AF_INET | AF_INET6
        line = line + ("family=%u " % (rtm._family,))
        # destination CIDR, eg. 24 or 32 for IPv4
        line = line + ("dst_len=%u " % (rtm._dst_len,))
        # source CIDR
        line = line + ("src_len=%u " % (rtm._src_len,))
        # type of service (TOS), eg. 0
        line = line + ("tos=%u " % (rtm._tos,))
        # table id
        line = line + ("table=%u " % (rtm._table,))
        # type
        line = line + ("type=%u " % (rtm._type,))
        # scope
        line = line + ("scope=%u " % (rtm._scope,))
        # protocol
        line = line + ("proto=%u " % (rtm._protocol,))
        # flags
        line = line + ("flags=%x" % (rtm._flags,))
        print(line)

        route_parser = RouteParser()
        attrs = route_parser.parse(msg.get_payload(), len(rtm))
        line = ""
        try:
            line = line + ("table=%u " % (attrs['table'],))
        except KeyError:
            pass
        try:
            line = line + ("dst=%s " % (attrs['dst'],))
        except KeyError:
            pass
        try:
            line = line + ("src=%s " % (attrs['src'],))
        except KeyError:
            pass
        try:
            line = line + ("oif=%u " % (attrs['oif'],))
        except KeyError:
            pass
        try:
            line = line + ("flow=%u " % (attrs['flow'],))
        except KeyError:
            pass
        try:
            line = line + ("prefsrc=%s " % (attrs['prefsrc'],))
        except KeyError:
            pass
        try:
            line = line + ("gw=%s " % (attrs['gateway'],))
        except KeyError:
            pass
        print(line)




#!/usr/bin/python
#
# attributes.py -- interface to netlink message payload attributes
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#
# Much of the method docstrings are from libmnl and are
#      Copyright 2008-2010 by Pablo Neira Ayuso <pablo@netfilter.org>
#

from struct import calcsize, pack, unpack

import pymnl

# Forward compatibility with Py3k
# copied from http://wiki.python.org/moin/PortingPythonToPy3k
try:
    bytes
except NameError:
    bytes = str


# netlink attributes
#
#  nla_type (16 bits)
#  +---+---+-------------------------------+
#  | N | O | Attribute Type                |
#  +---+---+-------------------------------+
#  N:=Carries nested attributes
#  O:=Payload stored in network byte order
#
#  Note:The N and O flag are mutually exclusive.
#
NLA_F_NESTED = (1 << 15)
NLA_F_NET_BYTEORDER = (1 << 14)
NLA_TYPE_MASK = ~(NLA_F_NESTED | NLA_F_NET_BYTEORDER)

NLA_ALIGNTO = 4
NLA_ALIGN = pymnl.PYMNL_ALIGN(NLA_ALIGNTO)

# pack/unpack format for type and length
header_format = "hh"

ATTR_HDRLEN = NLA_ALIGN(calcsize(header_format))

# minimal netlink attribute types
TYPE_UNSPEC = 0         # Unspecified type
TYPE_U8 = 1             # 8bit integer
TYPE_U16 = 2            # 16bit integer
TYPE_U32 = 3            # 32bit integer
TYPE_U64 = 4            # 64bit integer
TYPE_STRING = 5         # character string
TYPE_FLAG = 6           # flag
TYPE_MSECS = 7          # micro seconds (64bit)
TYPE_NESTED = 8         # nested attributes
TYPE_NESTED_COMPAT = 9
TYPE_NUL_STRING = 10    # null-terminated character string
TYPE_BINARY = 11
TYPE_MAX = 12           # always keep last

# various data type sizes
_u8 = calcsize("B")
_u16 = calcsize("H")
_u32 = calcsize("I")
_u64 = calcsize("Q")


class Attr(object):
    """ Netlink Length-Type-Value (LTV) attribute:

        |<-- 2 bytes -->|<-- 2 bytes -->|<-- variable -->|
        -------------------------------------------------
        |     length    |      type     |      value     |
        -------------------------------------------------
        |<--------- header ------------>|<-- payload --->|

        The payload of the Netlink message contains sequences of
        attributes that are expressed in LTV format.

        Attr stores the value in packed format, internally.  The value
        is immediately packed when an Attr is created via non-packed data.
        It will be unpacked, as needed, when called for through the
        get_*() methods.
    """
    def __init__(self, type=None, value=None, packed_data=None):
        """ Create a new Attr object.

            type - attribute's type (see NLA_* constants in linux/netlink.h)

            value - string or number representing the payload

            packed_data - This is the binary string containing the
                attribute data.  This could be the output from another
                object's get_binary() method or data directly from Netlink.
        """
        if (packed_data):
            # process packed struct into Attr's fields
            (self._length,
             self._type) = unpack(header_format, packed_data[:4])
            self._value = packed_data[4:]
        else:
            self.set(type, value)


    def __len__(self):
        """ Get the length of the packed attribute (in bytes).
        """
        return ATTR_HDRLEN + NLA_ALIGN(self.get_value_len())

    def get_value_len(self):
        """ Return the length of the data payload.
        """
        return len(self._value)

    @staticmethod
    def new_u8(type, value):
        """ Return a new one byte long Attr object.
        """
        if ((value < 0) or (value > pow(2, 8))):
            raise TypeError
        return Attr(type=type, value=pack("B", value))

    @staticmethod
    def new_u16(type, value):
        """ Return a new two byte long Attr object.
        """
        if ((value < 0) or (value > pow(2, 16))):
            raise TypeError
        return Attr(type=type, value=pack("H", value))

    @staticmethod
    def new_u32(type, value):
        """ Return a new four byte long Attr object.
        """
        if ((value < 0) or (value > pow(2, 32))):
            raise TypeError
        return Attr(type=type, value=pack("I", value))

    @staticmethod
    def new_u64(type, value):
        """ Return a new eight byte long Attr object.
        """
        if ((value < 0) or (value > pow(2, 64))):
            raise TypeError
        return Attr(type=type, value=pack("Q", value))

    @staticmethod
    def new_strnz(type, value):
        """ Return a new Attr object with a non-zero-terminated string.
        """
        if (not isinstance(value, bytes)):
            raise TypeError
        return Attr(type=type, value=pack(repr(len(value)) + "s", value))

    @staticmethod
    def new_strz(type, value):
        """ Return a new Attr object with a zero-terminated string.

            This method will add the null termination.  Pass this
            method a non-zero-terminated string.
        """
        if (not isinstance(value, bytes)):
            raise TypeError
        value = value + b'\x00'
        return Attr.new_strnz(type=type, value=value)

    def set(self, type, value):
        """ Set the attribute type and value.

            type - attribute's type (see NLA_* constants in linux/netlink.h)

            value - string representing the payload
        """
        self._type = type
        self._value = value

    def get_type(self):
        """ Get the attribute's type.
        """
        return self._type & NLA_TYPE_MASK

    def type_valid(self):
        """ Return False if the type is not known, otherwise return True.
        """
        if (self.get_type() > TYPE_MAX):
            return False
        return True

    def get_u8(self):
        """ Return value as a one byte integer.

            Raises TypeError if the data length does not match
            the length expected for the type.
        """
        if (self.get_type() == TYPE_U8 and self.get_value_len() != _u8):
            raise TypeError("Integer attribute is too short")
        return unpack("B", self._value)[0]

    def get_u16(self):
        """ Return value as a two byte integer.

            Raises TypeError if the data length does not match
            the length expected for the type.
        """
        if (self.get_type() == TYPE_U16 and self.get_value_len() != _u16):
            raise TypeError("Integer attribute is too short")
        return unpack("H", self._value)[0]

    def get_u32(self):
        """ Return value as a four byte integer.

            Raises TypeError if the data length does not match
            the length expected for the type.
        """
        if (self.get_type() == TYPE_U32 and self.get_value_len() != _u32):
            raise TypeError("Integer attribute is too short")
        return unpack("I", self._value)[0]

    def get_u64(self):
        """ Return value as an eight byte integer.

            Raises TypeError if the data length does not match
            the length expected for the type.
        """
        if (self.get_type() == TYPE_U64 and self.get_value_len() != _u64):
            raise TypeError("Integer attribute is too short")
        return unpack("Q", self._value)[0]

    def get_str(self):
        """ Return value as a string.

            Raises TypeError if the data length is zero or is
            not null-terminated.  A null-terminated string
            should have a non-zero length and a null termination
            or something went wrong.
        """
        if (type == TYPE_NUL_STRING):
            if (self.get_value_len() == 0):
                raise TypeError("String attribute is too short")
            if (self._value[-1:] != 0):
                raise TypeError("This attribute is not null-terminated," +
                                "as it claims to be")
        return unpack(repr(len(self._value)) + "s", self._value)[0]

    def get_str_stripped(self):
        """ Return value as a string, without zero terminator.

            Raises TypeError if the data length is zero.
            A string should have a non-zero length or something went wrong.
        """
        if (type == TYPE_STRING and self.get_value_len() == 0):
            raise TypeError("String attribute is too short")
        string_ = self.get_str()
        if ((string_[-1] == b'\x00') or (string_[-1] == 0)):
            # b'\x00' works in Py2, but 0 works in Py3, gah!
            string_ = string_[:-1]
        return string_

    def get_binary(self):
        """ Return a packed struct to include in message payload.

            Adds null-bytes to end to pad attribute's length to
            a multiple of NLA_ALIGNTO.
        """
        # prepare the header info
        header = pack(header_format, len(self), self._type)
        # prepare the null padding
        pad = (NLA_ALIGN(len(self._value)) - self.get_value_len()) * b'\x00'
        # push the whole package out
        return header + self._value + pad

    def get_data(self):
        """ Return the non-header data string, a.k.a. the attribute's
            payload (not it's Payload).
        """
        return self._value

    def toggle_nested(self):
        """ Toggle the nested flag on and off for the Attr object.
        """
        self._type = self._type ^ NLA_F_NESTED

    def is_nested(self):
        """ Return True if the nested flag is set, else return False.
        """
        return self._type & NLA_F_NESTED


class AttrParser(object):
    """ Base class for attribute parsers.

        This class provides the most basic parsing capability.  In most
        cases, you should use a subclass with callback methods or even
        replace the parse() method.

        However, AttrParser will handle simple attribute data.  And return
        a list of the attributes found.
    """
    def __init__(self, data_obj=None):
        """ Parse a string for netlink attributes.

            data_obj - An optional object with attributes.  The data
                object can be passed here and will be immediately parsed.
                Or the object can be sent to the parse() method after
                initialization.  See parse() for more details.
        """
        # list to hold attributes if no callbacks are assigned
        self._attributes = []
        # dict to hold attribute type to callback method mapping
        self._cb = {}
        if (data_obj):
            self._attributes = self.parse(data_obj)

    def parse_string(self, data, offset=0):
        """ Process the attributes.

            data - raw data to parse

            offset - offset into data at which to start

            This method takes care of the low-level detail of finding
            attributes in a data string and returns individual
            attributes one at a time (parse_string() is a generator).

            Subclasses can override parse() and still use parse_string()
            to retrieve individual attributes from the data.
        """
        index = offset
        while (index < len(data)):
            try:
                attr_length = unpack("H", data[index:index+2])[0]
            except:
                break
            end_index = index + attr_length
            one_attr = Attr(packed_data=data[index:end_index])
            index = NLA_ALIGN(end_index)
            yield one_attr

    def parse(self, data_obj):
        """ Returns a list of Attr processed from the binary string.

            data_obj - An object containing attributes and providing the
                get_binary() method.  See Message and Payload for examples
                of get_binary().
        """
        attributes = []
        for one_attr in self.parse_string(data_obj.get_binary()):
            try:
                self._cb[one_attr.get_type()](one_attr)
            except KeyError:
                attributes.append(one_attr)
        if (len(attributes) > 0):
            return attributes

    def parse_nested(self, data_obj):
        """ Returns a list of Attr processed from the binary string.

            data_obj - An object containing nested attributes and providing
                the get_data() method.  get_data() must return the non-header
                binary string to be parsed here.  See Payload and Attr
                for examples of get_data().
        """
        attributes = []
        for one_attr in self.parse_string(data_obj.get_data(), 0):
            attributes.append(one_attr)
        if (len(attributes) > 0):
            return attributes

    def get_attrs(self):
        """ Return list of attributes parsed from data string.

              If the data string is passed on object creation, and no
              callback methods have been assigned, this method can be
              used to get the list of attributes.
        """
        return self._attributes




#!/usr/bin/python
# tests/attributes.py -- test interface for netlink message attributes
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from random import randint
import unittest

import pymnl
from pymnl.attributes import *

class TestAttributes(unittest.TestCase):

    def _test_integer_constructor(self, const_, max_value_):
        """ Test the specified integer constructor.

            const_ - Attr constructor to test

            max_value_ - maximum value the integer can hold
        """
        for type_ in (TYPE_U8, TYPE_U16, TYPE_U32, TYPE_U64):
            # valid values
            random_ = randint(1, max_value_ - 1)
            for value_ in (0, random_, max_value_):
                self.assertTrue(isinstance(const_(type_, value_), Attr),
                                        "test value did not make an Attr")
            # invalid values
            random_ = randint(-1 * (max_value_ - 1), -2)
            for value_ in (-1, random_, -1 * max_value_, "test string"):
                self.assertRaises(TypeError, const_, type_, value_)

    def _test_integer_length(self, const_, max_value_, aligned_len_):
        """ Test the specified integer length.

            const_ - Attr constructor

            max_value_ - maximum value the integer can hold

            aligned_len_ - length of Attr value aligned to NLA size (in
                bytes); this value should not include the attribute header
        """
        for type_ in (TYPE_U8, TYPE_U16, TYPE_U32, TYPE_U64):
            random_ = randint(1, max_value_ - 1)
            for value_ in (0, random_, max_value_):
                one_attr = const_(type_, value_)
                # valid values
                self.assertEqual(len(one_attr), ATTR_HDRLEN + aligned_len_,
                            "length does not match expected Attr length")
                # invalid values
                self.assertNotEqual(len(const_(type_, value_)),
                            ATTR_HDRLEN + randint(0, aligned_len_ - 1),
                            "length unexpectedly matches")

    def _test_type(self, const_, value_):
        """ Test the Attr type.

            const_ - Attr constructor

            value_ - value for the Attr
        """
        # valid values
        for type_ in (TYPE_U8, TYPE_U16, TYPE_U32, TYPE_U64,
                                         TYPE_STRING, TYPE_NUL_STRING):
            one_attr = const_(type_, value_)
            self.assertEqual(one_attr.get_type(), type_,
                                    "returned type did not match")
            self.assertTrue(one_attr.type_valid(), "invalid type")
        # invalid values
        one_attr = const_(TYPE_MAX + 1, value_)
        self.assertFalse(one_attr.type_valid(), "unexpectedly valid type")

    def _test_integer_return(self, const_, get_method_, max_value_):
        """ Test an Attr.get_*() method.
        """
        for type_ in (TYPE_U8, TYPE_U16, TYPE_U32, TYPE_U64):
            # valid values
            random_ = randint(1, max_value_ - 1)
            for value_ in (0, random_, max_value_):
                one_attr = const_(type_, value_)
                self.assertEqual(get_method_(one_attr), value_,
                            "returned value does not match entered value")

    def _test_get_binary(self, const_, type_, value_, expected_value_):
        """ Test Attr.get_binary() method.
        """
        one_attr = const_(type_, value_)
        self.assertEqual(one_attr.get_binary(), expected_value_,
                    "binary value does not match expected value")

    def test_u8(self):
        """ Test one byte long Attr objects.
        """
        max_value_ = pow(2, 8) - 1
        self._test_integer_constructor(Attr.new_u8, max_value_)
        self._test_integer_length(Attr.new_u8, max_value_, 4)
        self._test_type(Attr.new_u8, 0)
        self._test_integer_return(Attr.new_u8, Attr.get_u8, max_value_)
        self._test_get_binary(Attr.new_u8, TYPE_U8, max_value_,
                                b'\x08\x00\x01\x00\xff\x00\x00\x00')

    def test_u16(self):
        """ Test two byte long Attr objects.
        """
        max_value_ = pow(2, 16) - 1
        self._test_integer_constructor(Attr.new_u16, max_value_)
        self._test_integer_length(Attr.new_u16, max_value_, 4)
        self._test_type(Attr.new_u16, 0)
        self._test_integer_return(Attr.new_u16, Attr.get_u16, max_value_)
        self._test_get_binary(Attr.new_u16, TYPE_U16, max_value_,
                                b'\x08\x00\x02\x00\xff\xff\x00\x00')

    def test_u32(self):
        """ Test four byte long Attr objects.
        """
        max_value_ = pow(2, 32) - 1
        self._test_integer_constructor(Attr.new_u32, max_value_)
        self._test_integer_length(Attr.new_u32, max_value_, 4)
        self._test_type(Attr.new_u32, 0)
        self._test_integer_return(Attr.new_u32, Attr.get_u32, max_value_)
        self._test_get_binary(Attr.new_u32, TYPE_U32, max_value_,
                                b'\x08\x00\x03\x00\xff\xff\xff\xff')

    def test_u64(self):
        """ Test eight byte long Attr objects.
        """
        max_value_ = pow(2, 64) - 1
        self._test_integer_constructor(Attr.new_u64, max_value_)
        self._test_integer_length(Attr.new_u64, max_value_, 8)
        self._test_type(Attr.new_u64, 0)
        self._test_integer_return(Attr.new_u64, Attr.get_u64, max_value_)
        self._test_get_binary(Attr.new_u64, TYPE_U64, max_value_,
                        b'\x0c\x00\x04\x00\xff\xff\xff\xff\xff\xff\xff\xff')

    def test_strnz(self):
        """ Test string value Attr objects.
        """
        for type_ in (TYPE_U8, TYPE_STRING):
            # valid values
            for test_string_ in (b'test string', b'nl80211', b'spam'):
                aligned_len_ = NLA_ALIGN(len(test_string_))
                strnz_ = Attr.new_strnz(type_, test_string_)
                self.assertTrue(isinstance(strnz_, Attr),
                                        "test string did not make an Attr")
                self.assertEqual(len(strnz_), (ATTR_HDRLEN + aligned_len_),
                                        "test string is wrong length")
                self.assertEqual(strnz_.get_str(), test_string_,
                            "returned value does not match entered value")
                self.assertEqual(strnz_.get_str_stripped(), test_string_,
                            "returned value does not match entered value")
            # invalid values
            random_ints_ = []
            for i_ in range(3): random_ints_.append(randint(-1000, 1000))
            for value_ in (random_ints_):
                self.assertRaises(TypeError, Attr.new_strnz, type_, value_)
        # other tests
        self._test_type(Attr.new_strnz, b'test')
        self._test_get_binary(Attr.new_strnz, TYPE_STRING, b'spam',
                                                b'\x08\x00\x05\x00spam')

    def test_strz(self):
        """ Test null-terminated string value Attr objects.
        """
        for type_ in (TYPE_U8, TYPE_NUL_STRING):
            # valid values
            for test_string_ in (b'test string', b'nl80211', b'spam'):
                aligned_len_ = NLA_ALIGN(len(test_string_) + 1)
                strz_ = Attr.new_strz(type_, test_string_)
                self.assertTrue(isinstance(strz_, Attr),
                                        "test string did not make an Attr")
                self.assertEqual(len(strz_), (ATTR_HDRLEN + aligned_len_),
                                        "test string is wrong length")
                self.assertEqual(strz_.get_str(), test_string_ + b'\x00',
                            "returned value does not match entered value")
                self.assertEqual(strz_.get_str_stripped(), test_string_,
                            "returned value does not match entered value")
            # invalid values
            random_ints_ = []
            for i_ in range(3): random_ints_.append(randint(-1000, 1000))
            for value_ in (random_ints_):
                self.assertRaises(TypeError, Attr.new_strz, type_, value_)
        # other tests
        self._test_type(Attr.new_strz, b'test')
        self._test_get_binary(Attr.new_strz, TYPE_NUL_STRING, b'spam',
                                b'\x0c\x00\n\x00spam\x00\x00\x00\x00')

    def test_nested(self):
        """ Test nested flag toggle and check.
        """
        one_attr = Attr.new_u8(TYPE_U8, 0)
        self.assertFalse(one_attr.is_nested(),
                    "nested flag should not be set in new Attr objects")
        one_attr.toggle_nested()
        self.assertTrue(one_attr.is_nested(),
                                            "nested flag should be set")
        one_attr.toggle_nested()
        self.assertFalse(one_attr.is_nested(),
                                        "nested flag should not be set")

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestAttributes)


class TestAttrParser(unittest.TestCase):

    def test_init_no_data(self):
        """ Test AttrParser object creation without data.
        """
        attr_parser = AttrParser()
        self.assertTrue(isinstance(attr_parser, AttrParser))

    def test_init_with_data(self):
        """ Test AttrParser object creation with data.
        """
        # manually create a TYPE_U16 attribute
        one_attr = Attr.new_u16(TYPE_U16, 4881)
        attr_parser = AttrParser(one_attr)
        self.assertTrue(isinstance(attr_parser, AttrParser))

    def test_parse_string(self):
        """ Test AttrParser.parse_string().
        """
        # no test, yet
        pass

    def test_parse(self):
        """ Test AttrParser.parse().
        """
        # manually create a TYPE_U16 attribute
        one_attr = Attr.new_u16(TYPE_U16, 4881)
        # create AttrParser without any data, yet
        attr_parser = AttrParser()
        # ask the parser to parse the test Attr
        attr_list = attr_parser.parse(one_attr)
        self.assertEqual(one_attr.get_binary(), attr_list[0].get_binary())

    def test_parse_nested(self):
        """ Test AttrParser.parse_nested().
        """
        # no test, yet
        pass

    def test_get_attrs(self):
        """ Test AttrParser.get_attrs().
        """
        # manually create a TYPE_U16 attribute
        one_attr = Attr.new_u16(TYPE_U16, 4881)
        # create AttrParser object using the test Attr
        attr_parser = AttrParser(one_attr)
        attr_list = attr_parser.get_attrs()
        self.assertEqual(one_attr.get_binary(), attr_list[0].get_binary())

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestAttrParser)




#!/usr/bin/python
#
# rtnl-link-set.py -- set an interface up or down
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

from random import randint
import socket
import sys

import pymnl
from pymnl.attributes import Attr
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

import if_
import if_link
import rtnetlink

change = 0
flags = 0

if (len(sys.argv) != 3):
    print("Usage: %s [ifname] [up|down]" % (sys.argv[0],))
    sys.exit()

if (sys.argv[2].lower() == "up"):
    change |= if_.IFF_UP
    flags |= if_.IFF_UP
elif (sys.argv[2].lower() == "down"):
    change |= if_.IFF_UP;
    flags &= ~if_.IFF_UP;
else:
    print("%s is not `up' nor `down'" % (sys.argv[2],))
    sys.exit()

# init and build request message
rtnlmsg = Message()
rtnlmsg.set_type(rtnetlink.RTM_NEWLINK)
rtnlmsg.set_flags(pymnl.message.NLM_F_REQUEST | pymnl.message.NLM_F_ACK)

sequence = randint(0, pow(2, 31))
rtnlmsg.set_seq(sequence)

# build ifm header and add it to the message
ifm = if_.IfInfoMessage()
ifm.family = socket.AF_UNSPEC
ifm.change = change
ifm.flags = flags

rtnlmsg.put_extra_header(ifm)

payload = rtnlmsg.get_payload()
payload.add_attr(Attr.new_strnz(if_link.IFLA_IFNAME, sys.argv[1].encode()))

# init and bind netlink socket
sock = Socket(pymnl.NETLINK_ROUTE)
sock.bind(pymnl.nlsocket.SOCKET_AUTOPID, 0)

rtnlmsg.printf(len(ifm))

# send message through socket
sock.send(rtnlmsg)

# read returned message from netlink
return_msg = sock.recv(flags=socket.MSG_DONTWAIT)

sock.close()

# process the messages returned
for msg in return_msg:
    if (msg.get_errno()):
        # tell the user what error occurred
        print("error:", msg.get_errstr())





#!/usr/bin/python
# tests/genl.py -- tests generic netlink header and parsers
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

import pickle
import random
from struct import pack
import unittest

import pymnl
import pymnl.genl

class TestGenl(unittest.TestCase):

    def test_genlmsgheader(self):
        """ Test genetlink message header.
        """
        genlmh = pymnl.genl.GenlMessageHeader(
                                command=pymnl.genl.CTRL_CMD_GETFAMILY,
                                version=1)
        binary = pack("BBH", 3, 1, 0)
        self.assertEqual(genlmh.get_binary(), binary)

    def test_parsers(self):
        """ Test the various genetlink parsers.
        """
        # load pickled genetlink message
        f = open('pymnl/tests/genl-test_msg.pickled', 'rb')
        test_msg = pickle.load(f)
        f.close()
        # load pickled processed genetlink payload
        f = open('pymnl/tests/genl-test_attrs.pickled', 'rb')
        test_attrs = pickle.load(f)
        f.close()
        # process the test message
        genl_parser = pymnl.genl.GenlAttrParser()
        try:
            attrs = genl_parser.parse(test_msg.get_payload())
        except TypeError:
            # Unpickling a Py2 object with Py3 causes weirdness,
            payload = test_msg.get_payload()
            # so reach into the message payload and encode
            # the string as a bytes.
            payload._contents = payload._contents.encode()
            # Now we can parse the message payload for attributes.
            attrs = genl_parser.parse(payload)
            # convert expected bytes values to string
            attrs['name'] = attrs['name'].decode()
            attrs['groups'][1] = attrs['groups'][1].decode()
            attrs['groups'][2] = attrs['groups'][2].decode()
            attrs['groups'][3] = attrs['groups'][3].decode()
            attrs['groups'][4] = attrs['groups'][4].decode()
        # check that the pickled payload matches the processed payload
        self.assertEqual(test_attrs, attrs)

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestGenl)




#!/usr/bin/python
#
# rtnl-link-event.py -- listen for and report link events
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

from random import randint
import socket
import sys

import pymnl
from pymnl.attributes import Attr
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

import if_
import if_link
import rtnetlink

# add a name for the multicast group on which to listen
RTMGRP_LINK = 1

# init and bind netlink socket
sock = Socket(pymnl.NETLINK_ROUTE)
sock.bind(pymnl.nlsocket.SOCKET_AUTOPID, RTMGRP_LINK)

try:
    while (True):
        # read returned message from netlink
        return_msg = sock.recv()

        # process the messages returned
        for msg in return_msg:
            if (msg.get_errno()):
                # tell the user what error occurred
                print("error:", msg.get_errstr())
            else:
                # use the payload data to create interface info message
                ifm = if_.IfInfoMessage(msg.get_payload().get_binary())
                # begin output line with interface info
                line = ("index=%d type=%d flags=%d family=%d " %
                        (ifm.index, ifm.type_, ifm.flags, ifm.family))

                # add running status to output line
                if (ifm.flags & if_.IFF_RUNNING):
                    line = line + "[RUNNING] "
                else:
                    line = line + "[NOT RUNNING] "

                ifla_parser = if_link.IFLAttrParser()
                # make a payload with the interface link attributes data
                ifla_payload = Payload(msg.get_payload().get_binary()[len(ifm):])
                # parse the new payload into a dict of Attrs
                attrs = ifla_parser.parse(ifla_payload)

                # add final interface info to output line
                line = line + ("mtu=%d name=%s " %
                                        (attrs['mtu'], attrs['ifname']))

                # finally output the dang line
                print(line)
except KeyboardInterrupt:
    sock.close()




#!/usr/bin/python
# tests/message.py -- test interface for netlink messages
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from random import randint
from struct import pack
import unittest

import pymnl
from pymnl.message import *

class TestPayload(unittest.TestCase):

    def _test_init(self):
        """ Test init of a Payload.
        """
        self.payload = Payload(pack("BBH", 3, 1, 0))
        self.binary = pack("ssss", "\x03", "\x01", "\x00", "\x00")
        self.assertEqual(self.payload.get_binary(), self.binary)

    def _test_add_attr(self):
        """ Test adding Attr objects to the Payload.
        """
        self.family_type = Attr.new_u32(1, 16)
        self.payload.add_attr(self.family_type)
        self.binary = self.binary + pack("ssssssss",
                                            "\x08", "\x00", "\x01", "\x00",
                                            "\x10", "\x00", "\x00", "\x00")
        self.assertEqual(self.payload.get_binary(), self.binary)

        self.family_name = Attr.new_strz(2, b'nl80211')
        self.payload.add_attr(self.family_name)
        self.binary = self.binary + pack("ssssssssssss",
                                            "\x0c", "\x00", "\x02", "\x00",
                                            "n", "l", "8", "0",
                                            "2", "1", "1", "\x00")
        self.assertEqual(self.payload.get_binary(), self.binary)

    def test_payload(self):
        """ Call private test methods which build a Payload step-by-step.
        """
        self._test_init()
        self._test_add_attr()

    def tearDown(self):
        """ Clean up after each test.
        """
        self.payload = None

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestPayload)


class TestMessage(unittest.TestCase):

    def setUp(self):
        """ Set up the initial conditions for each test.
        """
        self.msg = Message()

    def _setUp(self):
        """ Set up the initial conditions for some tests.

            Creates hypothetical genl protocol for more complicated tests.
        """
        self.msg._msg_type = 16  # GENL_ID_CTRL
        self.msg._msg_flags = 5  # NLM_F_REQUEST | NLM_F_ACK

        self.seq = randint(0, pow(2, 31))
        self.msg._msg_seq = self.seq

        self.pid = randint(0, pow(2, 31))
        self.msg._pid = self.pid

        self.msg_length = 16
        self.msg_header = pack("HHII", self.msg._msg_type,
                                       self.msg._msg_flags,
                                       self.seq, self.pid)

    def _test_valid_header_values(self, set_method_, get_method_,
                                min_value_, max_value_):
        """ Test a Message header accessor with valid values.
        """
        random_ = randint(min_value_ + 1, max_value_ - 1)
        for value_ in (min_value_, random_, max_value_):
            set_method_(self.msg, value_)
            self.assertEqual(get_method_(self.msg), value_,
                            "returned value does not match entered value")

    def _test_invalid_header_values(self, set_method_, get_method_,
                                        min_value_, max_value_):
        """ Test a Message header accessor with invalid values.
        """
        random_ = randint(min_value_ + 1, max_value_ - 1)
        for value_ in (min_value_, random_, max_value_):
            self.assertRaises(ValueError, set_method_, self.msg, value_)

    def test_type(self):
        """ Test valid and invalid type values.
        """
        self._test_valid_header_values(Message.set_type, Message.get_type,
                                        0, 0xffff)
        self._test_invalid_header_values(Message.set_type, Message.get_type,
                                            0xffff + 1, 0xffffffff)

    def test_flags(self):
        """ Test valid and invalid flags values.
        """
        self._test_valid_header_values(Message.set_flags, Message.get_flags,
                                        0, 0xffff)
        self._test_invalid_header_values(Message.set_flags, Message.get_flags,
                                            0xffff + 1, 0xffffffff)

    def test_seq(self):
        """ Test valid and invalid sequence numbers.
        """
        self._test_valid_header_values(Message.set_seq, Message.get_seq,
                                        0, 0xffffffff)
        self._test_invalid_header_values(Message.set_seq, Message.get_seq,
                                        0xffffffff + 1, 0xffffffffffffffff)

    def test_portid(self):
        """ Test valid and invalid portid values.
        """
        self._test_valid_header_values(Message.set_portid,
                                        Message.get_portid,
                                        0, 0xffffffff)
        self._test_invalid_header_values(Message.set_portid,
                                         Message.get_portid,
                                         0xffffffff + 1, 0xffffffffffffffff)

    def test_put_extra_header(self):
        """ Test Message.put_extra_header().
        """
        self._setUp()
        # add a four byte header object
        extra_header = Payload(pack("BBH", 3, 1, 0))
        self.msg.put_extra_header(extra_header)
        self.msg_length = self.msg_length + 4
        self.binary = (pack("I", self.msg_length) + self.msg_header +
                        extra_header.get_binary())
        self.assertEqual(self.msg.get_binary(), self.binary)

    def test_add_payload(self):
        """ Test Message.add_payload().
        """
        self._setUp()
        payload = Payload(pack("BBH", 3, 1, 0))
        self.msg.add_payload(payload)
        self.msg_length = self.msg_length + 4
        self.binary = (pack("I", self.msg_length) + self.msg_header +
                        payload.get_binary())
        self.assertEqual(self.msg.get_binary(), self.binary)

    def test_get_payload(self):
        """ Test Message.get_payload().

            In reality, this justs tests that all the names for the same
            object point to the same object.  I.E. this is not much of
            a test.
        """
        self._setUp()
        payload_in = Payload(pack("BBH", 3, 1, 0))
        self.msg.add_payload(payload_in)
        payload_out = self.msg.get_payload()
        self.assertEqual(payload_in, payload_out)

    def test_ok(self):
        """ Test Message.ok().
        """
        self._setUp()
        self.assertTrue(self.msg.ok())

    def test_seq_ok(self):
        """ Test Message.seq_ok().

            Check that initial sequence number is still used and that
            a random integer does not match the sequence number.
        """
        self._setUp()
        self.assertTrue(self.msg.seq_ok(self.seq))
        # false result possible if random number == seq
        self.assertFalse(self.msg.seq_ok(randint(0, pow(2, 31))))

    def test_portid_ok(self):
        """ Test Message.portid_ok().

            Check that initial portid number is still used and that
            a random integer does not match the portid number.  In
            real-world use, generally, portid is sent as 0 and is
            assigned another number by the kernel.
        """
        self._setUp()
        self.assertTrue(self.msg.portid_ok(self.pid))
        # false result possible if random number == portid
        self.assertFalse(self.msg.portid_ok(randint(0, pow(2, 31))))

    def test_get_errno(self):
        """ Test Message.get_errno().
        """
        self._setUp()
        # no error code
        payload = Payload(pack("i", 0))
        self.msg.add_payload(payload)
        self.msg_length = self.msg_length + 4
        self.assertEqual(self.msg.get_errno(), 0)
        # ENOENT error
        self.msg._msg_type = 0x2   # NLMSG_ERROR
        payload = Payload(pack("i", -2))
        self.msg._payload = payload
        self.assertEqual(self.msg.get_errno(), 2)

    def test_get_errstr(self):
        """ Test Message.get_errstr().
        """
        self._setUp()
        # no error code
        payload = Payload(pack("i", 0))
        self.msg.add_payload(payload)
        self.msg_length = self.msg_length + 4
        self.assertEqual(self.msg.get_errstr(), 'Success')
        # ENOENT error
        self.msg._msg_type = 0x2   # NLMSG_ERROR
        payload = Payload(pack("i", -2))
        self.msg._payload = payload
        self.assertEqual(self.msg.get_errstr(), 'No such file or directory')

    def tearDown(self):
        """ Clean up after each test.
        """
        self.msg = None

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestMessage)


class TestMessageList(unittest.TestCase):

    def test_msglist(self):
        """
        """
        self.payload = Payload(b'\x03\x01\x00\x00\x08\x00\x01\x00\x10\x00\x00\x00\x0c\x00\x02\x00nl80211\x00')
        self.msg1 = Message()
        self.msg1.add_payload(self.payload)
        self.msg2 = Message()
        self.msg2.add_payload(self.payload)
        self.msg3 = Message()
        self.msg3.add_payload(self.payload)
        self.msg = (self.msg1.get_binary() + self.msg2.get_binary() +
                        self.msg3.get_binary())
        self.msglist = MessageList(self.msg)
        self.assertEqual(len(self.msglist), 3)

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestMessageList)




#!/usr/bin/python
# tests/nlsocket.py -- test interface to netlink socket
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

import random
import unittest

import pymnl
from pymnl.nlsocket import Socket

class TestSocket(unittest.TestCase):

    def setUp(self):
        """ Set up conditions necessary for each test.
        """
        self.nl_socket = Socket(pymnl.NETLINK_GENERIC)
        self._pid = random.randint(1024, 32768)
        self._groups = 0
        self.nl_socket.bind(self._pid, self._groups)

    def test_bus(self):
        """ Test that the requested bus is the actual bus.
        """
        self.assertEqual(self.nl_socket._bus, pymnl.NETLINK_GENERIC)

    def test_pid(self):
        """ Test that the requested port id is the actual port id.
        """
        self.assertEqual(self.nl_socket.get_portid(), self._pid)

    def test_groups(self):
        """ Test that the requested groups is the actual groups.
        """
        self.assertEqual(self.nl_socket.get_groups(), self._groups)

    def tearDown(self):
        """ Clean up after each test.
        """
        self.nl_socket.close()

    @staticmethod
    def suite():
        return unittest.TestLoader().loadTestsFromTestCase(TestSocket)





#!/usr/bin/python
#
# rtnl-link-dump3.py -- dump kernel routing info from rtnetlink
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

from random import randint
import socket
import sys

import pymnl
from pymnl.attributes import Attr
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

import if_
import if_link
import rtnetlink

# init and bind netlink socket
sock = Socket(pymnl.NETLINK_ROUTE)
sock.bind(pymnl.nlsocket.SOCKET_AUTOPID, 0)

# init and build request message
rtnlmsg = Message()
rtnlmsg.set_type(rtnetlink.RTM_GETLINK)
rtnlmsg.set_flags(pymnl.message.NLM_F_REQUEST | pymnl.message.NLM_F_DUMP)

sequence = randint(0, pow(2, 31))
rtnlmsg.set_seq(sequence)

# build rtgenmsg header and add it to the message
rtgenmsg_header = rtnetlink.RtGenMessageHeader(socket.AF_PACKET)
rtnlmsg.put_extra_header(rtgenmsg_header)

# send message through socket
sock.send(rtnlmsg)

# read returned message from netlink
return_msg = sock.recv(flags=socket.MSG_DONTWAIT)

sock.close()

# process the messages returned
for msg in return_msg:
    if (msg.get_errno()):
        # tell the user what error occurred
        print("error:", msg.get_errstr())
    else:
        # use the payload data to create interface info message
        ifm = if_.IfInfoMessage(msg.get_payload().get_binary())
        # begin output line with interface info
        line = ("index=%d type=%d flags=%d family=%d " %
                (ifm.index, ifm.type_, ifm.flags, ifm.family))

        # add running status to output line
        if (ifm.flags & if_.IFF_RUNNING):
            line = line + "[RUNNING] "
        else:
            line = line + "[NOT RUNNING] "

        ifla_parser = if_link.IFLAttrParser()
        # make a payload with the interface link attributes data
        ifla_payload = Payload(msg.get_payload().get_binary()[len(ifm):])
        # parse the new payload into a dict of Attrs
        attrs = ifla_parser.parse(ifla_payload)

        # add final interface info to output line
        line = line + ("name=%s mtu=%d " % (attrs['ifname'], attrs['mtu']))

        # finally output the dang line
        print(line)




#!/usr/bin/python
#
# if_.py -- constants and data structures from net/if.h
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from struct import calcsize, pack, unpack

IFF_UP = 0x1          # interface is up
IFF_RUNNING = 0x40

class IfInfoMessage(object):
    """ Passes link level specific information, not dependent on network
        protocol. -- /usr/include/linux/rtnetlink.h
    """
    def __init__(self, contents=None):
        """
        """
        self._format = "BBHiII"
        self.family = 0
        self.pad = 0
        self.type_ = 0
        self.index = 0
        self.flags = 0
        self.change = 0
        if (contents):
            contents = contents[:calcsize(self._format)]
            (self.family,
             self.pad,
             self.type_,
             self.index,
             self.flags,
             self.change) = unpack(self._format, contents)

    def __len__(self):
        """
        """
        return calcsize(self._format)

    def get_binary(self):
        """
        """
        return pack(self._format, self.family, self.pad, self.type_,
                        self.index, self.flags, self.change)




#!/usr/bin/python
#
# rtnetlink.py -- constants and data structures from linux/rtnetlink.h
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from struct import calcsize, pack, unpack

# Route message types
RTM_NEWLINK = 16
RTM_GETLINK = 18
RTM_NEWROUTE = 24
RTM_GETROUTE = 26

class RtGenMessageHeader(object):
    """ A Netlink route generic message header.
    """
    def __init__(self, family=None):
        """ Create a Netlink route generic message header.

            Objects of this type can be used with
            pymnl.message.put_extra_header().

            family - integer - the route message type
        """
        self._format = "B"
        self._family = family

    def set_family(self, family):
        """ Set the route message family.

            family - integer - the route message type
        """
        self._family = family

    def get_family(self):
        """ Returns the route message family.
        """
        return self._family

    def get_binary(self):
        """ Returns a packed struct suitable for sending through a
            netlink socket.

            Raises an exception if family has not been set before calling
            get_binary().
        """
        return pack(self._format, self._family)


# rtm_type
RTN_UNSPEC = 0
RTN_UNICAST = 1
RTN_LOCAL = 2
RTN_BROADCAST = 3
RTN_ANYCAST = 4
RTN_MULTICAST = 5
RTN_BLACKHOLE = 6
RTN_UNREACHABLE = 7
RTN_PROHIBIT = 8
RTN_THROW = 9
RTN_NAT = 10
RTN_XRESOLVE = 11
__RTN_MAX = 12

# rtm_protocol
RTPROT_UNSPEC = 0
RTPROT_REDIRECT = 1
RTPROT_KERNEL = 2   # route installed by kernel
RTPROT_BOOT = 3     # route installed during boot
RTPROT_STATIC = 4   # route installed by administrator
# Values >= RTPROT_STATIC are not interpreted by kernel, they are
# just user-defined.

# rtm_scope
#
# Really it is not scope, but sort of distance to the destination.
# NOWHERE are reserved for not existing destinations, HOST is our
# local addresses, LINK are destinations, located on directly attached
# link and UNIVERSE is everywhere in the Universe.
#
# Intermediate values are also possible f.e. interior routes
# could be assigned a value between UNIVERSE and LINK.
RT_SCOPE_UNIVERSE = 0     # everywhere in the universe
#    ... user defined values ...
RT_SCOPE_SITE = 200
RT_SCOPE_LINK = 253       # destination attached to link
RT_SCOPE_HOST = 254       # local address
RT_SCOPE_NOWHERE  = 255   # not existing destination

# rtm_flags
RTM_F_NOTIFY = 0x100    # notify user of route change
RTM_F_CLONED = 0x200    # this route is cloned
RTM_F_EQUALIZE = 0x400  # Multipath equalizer: NI
RTM_F_PREFIX = 0x800    # Prefix addresses

# Reserved table identifiers
RT_TABLE_UNSPEC = 0
# ... user defined values ...
RT_TABLE_COMPAT = 252
RT_TABLE_DEFAULT = 253
RT_TABLE_MAIN = 254
RT_TABLE_LOCAL = 255
RT_TABLE_MAX = 0xFFFFFFFF
# Synonimous attribute: RTA_TABLE.

# Routing message attributes
RTA_UNSPEC = 0
RTA_DST = 1
RTA_SRC = 2
RTA_IIF = 3
RTA_OIF = 4
RTA_GATEWAY = 5
RTA_PRIORITY = 6
RTA_PREFSRC = 7
RTA_METRICS = 8
RTA_MULTIPATH = 9
RTA_PROTOINFO = 10  # no longer used
RTA_FLOW = 11
RTA_CACHEINFO = 12
RTA_SESSION = 13  # no longer used
RTA_MP_ALGO = 14  # no longer used
RTA_TABLE = 15
RTA_MAX = 16

class RtMessage(object):
    """ A Netlink route message.
    """
    def __init__(self, family=0, dst_len=0, src_len=0, tos=0,
                       table=0, protocol=0, scope=0, type_=0,
                       flags=0, packed_data=None):
        """ Create a Netlink route message.

            Objects of this type can be used with
            pymnl.message.put_extra_header().

            family - unsigned char
            dst_len - unsigned char
            src_len - unsigned char
            tos - unsigned char

            table - unsigned char - Routing table id
            protocol - unsigned char - Routing protocol
            scope - unsigned char
            type_ - unsigned char

            flags - unsigned integer
        """
        self._format = "BBBBBBBBI"
        self._family = family
        self._dst_len = dst_len
        self._src_len = src_len
        self._tos = tos
        self._table = table
        self._protocol = protocol
        self._scope = scope
        self._type = type_
        self._flags = flags
        if (packed_data):
            packed_data = packed_data[:calcsize(self._format)]
            (self._family,
             self._dst_len,
             self._src_len,
             self._tos,
             self._table,
             self._protocol,
             self._scope,
             self._type,
             self._flags) = unpack(self._format, packed_data)

    def __len__(self):
        """
        """
        return calcsize(self._format)

    def get_binary(self):
        """ Returns a packed struct suitable for sending through a
            netlink socket.
        """
        return pack(self._format, self._family,
                                  self._dst_len,
                                  self._src_len,
                                  self._tos,
                                  self._table,
                                  self._protocol,
                                  self._scope,
                                  self._type,
                                  self._flags)




#!/usr/bin/python
# test -- implements a test command for distutils.core.setup
# Copyright (c) 2011 Sean Robinson <seankrobinson@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import distutils.core
import imp
import re
import unittest

class test(distutils.core.Command):
    """ Run unit tests.

        This command loads the modules specified with --test-list=
        (e.g. --test-list=tests.message,tests.attributes) and looks in
        each for class names starting with 'Test'.  This is similar to the
        way unittest.TestLoader looks for test cases as method names
        beginning with 'test'.

        This command then queries each found class for its test suite
        using an expected suite() static method which returns the test
        cases found by unittest.TestLoader().loadTestsFromTestCase(TestClass).

        Then the tests are run using unittest.TextTestRunner() with a
        default verbosity of 0, or a verbosity of 2 if the --test-verbose
        option is passed to the command.

        This class seems to work with Python > 2.6.
    """
    description = "Run unit tests."

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [('test-list=', None,
                     "list of test modules to run " +
                     "(e.g. tests.message)"),
                    ('test-verbose', None,
                     "show additional output during tests"),
                   ]

    boolean_options = ['test-verbose']

    def initialize_options(self):
        """ Set the default values for command options.
        """
        self.test_list = None
        self.test_verbose = 0

    def finalize_options(self):
        """ Ensure options are ready to be used during test set-up.
        """
        if self.test_list:
            self.test_list = re.split(",", self.test_list)
        else:
            self.test_list = []
        if (self.test_verbose):
            self.test_verbose = 2

    def load_module(self, mod_name, path=None):
        """ Recursively work down a module hierarchy to load the
            module at the bottom of the list.

            mod_name - string with the name of the module to load,
                        e.g. "tests.message" (see imp.find_module())

            path - list of directories to search (see imp.find_module())

            Returns an instance of the loaded module.
        """
        hierarchy = re.split("\.", mod_name)
        mod_specs = imp.find_module(hierarchy[0], path)
        try:
            module = imp.load_module(hierarchy[0], mod_specs[0],
                                                mod_specs[1], mod_specs[2])
        finally:
            if (mod_specs[0]):
                mod_specs[0].close()
        if (len(hierarchy) > 1):
            module = self.load_module(".".join(hierarchy[1:]),
                                                        module.__path__)
        return module

    def run(self):
        """ Find and load module(s) with test classes, ask those classes
            for a test suite, and run the returned tests.
        """
        test_class_pattern = re.compile("Test")
        for test_module in self.test_list:
            test_module = self.load_module(test_module)
            for module_attr in dir(test_module):
                result = test_class_pattern.match(module_attr)
                if (result):
                    class_ = test_module.__getattribute__(module_attr)
                    suite = class_.suite()
                    unittest.TextTestRunner(verbosity=self.test_verbose).run(suite)




#!/usr/bin/python

from distutils.core import setup

from pymnl.tests.testcommand import test

def get_version():
    version_file = open("docs/VERSION")
    try:
        version = version_file.readline()
    finally:
        version_file.close()
    return version[:-1]

setup(
    name = "pymnl",
    version = get_version(),
    author = "Sean Robinson",
    author_email = "seankrobinson@gmail.com",
    maintainer_email = "pymnl-dev@lists.tuxfamily.org",
    description = """pymnl (rhymes with hymnal) is a pure Python
re-implmentation of libmnl and provides a minimal interface to Linux
Netlink.""",
    url = "http://pymnl.wikispot.org/",
    packages = ['pymnl', 'pymnl.tests'],
    provides = ['pymnl'],

    platforms = "Linux",
    license = "LGPL for module; GPL for example scripts",
    keywords = "Netlink genl rtnl socket",
    download_url = "https://pymnl.tuxfamily.org",
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: System',
    ],
    cmdclass={'test': test},
)




#!/usr/bin/python
#
# message.py -- interface to netlink messages
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#
# Much of the method docstrings are from libmnl and are
#      Copyright 2008-2010 by Pablo Neira Ayuso <pablo@netfilter.org>
#

import os
from struct import calcsize, pack, unpack

import pymnl
from pymnl.attributes import Attr

# Flags values
NLM_F_REQUEST = 1       # It is a request message.
NLM_F_MULTI = 2         # Multipart message, terminated by NLMSG_DONE
NLM_F_ACK = 4           # Reply with ack, with zero or error code
NLM_F_ECHO = 8          # Echo this request

# Modifiers to GET request
NLM_F_ROOT = 0x100      # specify tree root
NLM_F_MATCH = 0x200     # return all matching
NLM_F_ATOMIC = 0x400    # atomic GET
NLM_F_DUMP = (NLM_F_ROOT|NLM_F_MATCH)

# Modifiers to NEW request
NLM_F_REPLACE = 0x100   # Override existing
NLM_F_EXCL = 0x200      # Do not touch, if it exists
NLM_F_CREATE = 0x400    # Create, if it does not exist
NLM_F_APPEND = 0x800    # Add to end of list

NLMSG_ALIGNTO = 4
NLMSG_ALIGN = pymnl.PYMNL_ALIGN(NLMSG_ALIGNTO)

NLMSG_NOOP = 0x1        # Nothing.
NLMSG_ERROR = 0x2       # Error
NLMSG_DONE = 0x3        # End of a dump
NLMSG_OVERRUN = 0x4     # Data lost

NLMSG_MIN_TYPE = 0x10   # < 0x10: reserved control messages

# pack/unpack format for msg_length, msg_type, msg_flags, msg_seq, pid
header_format = "ihhii"

MSG_HDRLEN = NLMSG_ALIGN(calcsize(header_format))

class Message(object):
    def __init__(self, buffer=None):
        """ A netlink message.

            Netlink message:

            |<----------------- 4 bytes ------------------->|
            |<----- 2 bytes ------>|<------- 2 bytes ------>|
            |-----------------------------------------------|
            |      Message length (including header)        |
            |-----------------------------------------------|
            |     Message type     |     Message flags      |
            |-----------------------------------------------|
            |           Message sequence number             |
            |-----------------------------------------------|
            |                 Netlink PortID                |
            |-----------------------------------------------|
            |                                               |
            .                   Payload                     .
            |_______________________________________________|

            There is usually an extra header after the the Netlink header
            (at the beginning of the payload). This extra header is specific
            of the Netlink subsystem. After this extra header, comes the
            sequence of attributes that are expressed in
            Length-Type-Value (LTV) format.
        """
        self._msg_length = 0
        self._msg_type = 0
        self._msg_flags = 0
        self._msg_seq = 0
        self._pid = 0
        self._payload = None

        if (buffer):
            (self._msg_length,
             self._msg_type,
             self._msg_flags,
             self._msg_seq,
             self._pid) = unpack(header_format, buffer[:MSG_HDRLEN])

            self._payload = Payload(buffer[NLMSG_ALIGN(MSG_HDRLEN):])

    def __len__(self):
        """ Get the unaligned length of the message (in bytes).
        """
        if (self._payload):
            return MSG_HDRLEN + len(self._payload)
        else:
            return MSG_HDRLEN

    def put_extra_header(self, header):
        """ Add a header before the Payload.

            header - The object which contains the extra header.  This
                        object must provide a get_binary() method which
                        returns a binary string/bytes with the netlink
                        data, like Payload or GenlMessageHeader.
        """
        if (self._payload):
            self._payload = Payload(header.get_binary() +
                                    self._payload.get_binary())
        else:
            self._payload = Payload(header.get_binary())

    def set_type(self, type_):
        """ Sets the message type.

            type_ - 2-byte integer
        """
        if ((type_ & 0xffff) != type_):
            raise ValueError("Message type must be between 0 and " + \
                                "65535, inclusive.")
        self._msg_type = type_

    def get_type(self):
        """ Returns the 2-byte type.
        """
        return self._msg_type

    def set_flags(self, flags_):
        """ Sets the message flags.

            flags_ - 2-byte integer
        """
        if ((flags_ & 0xffff) != flags_):
            raise ValueError("Message flags must be between 0 and " + \
                                "65535, inclusive.")
        self._msg_flags = flags_

    def get_flags(self):
        """ Returns the 2-byte flag.
        """
        return self._msg_flags

    def set_seq(self, seq_):
        """ Sets the message sequence number.

            seq_ - 4-byte integer
        """
        if ((seq_ & 0xffffffff) != seq_):
            raise ValueError("Message sequence number must be " + \
                                "between 0 and 4294967295, inclusive.")
        self._msg_seq = seq_

    def get_seq(self):
        """ Returns the 4-byte sequence number.
        """
        return self._msg_seq

    def set_portid(self, pid_):
        """ Sets the message port id.

            pid_ - 4-byte integer
        """
        if ((pid_ & 0xffffffff) != pid_):
            raise ValueError("Message port id must be between 0 and " + \
                                "4294967295, inclusive.")
        self._pid = pid_

    def get_portid(self):
        """ Returns the 4-byte port id.
        """
        return self._pid

    def add_payload(self, data):
        """ Add payload to the message.

            data - Payload or string
        """
        if (not isinstance(data, Payload)):
            data = Payload(data)
        if (self._payload):
            self._payload = Payload(self._payload.get_binary() +
                                        data.get_binary())
        else:
            self._payload = data

    def get_payload(self):
        """ Return the payload object contained in the message.
        """
        return self._payload

    def ok(self):
        """ Check that Message is internally consistent. (i.e. verify that
            a netlink message is not malformed nor truncated.
        """
        return len(self) >= MSG_HDRLEN

    def seq_ok(self, seq):
        """ Perform sequence tracking.

            seq - last sequence number used to send a message

            This method returns true if the sequence tracking is
            fulfilled, otherwise false is returned. We skip the tracking
            for netlink messages whose sequence number is zero since it is
            usually reserved for event-based kernel notifications. On the
            other hand, if seq is set but the message sequence number is
            not set (i.e. this is an event message coming from
            kernel-space), then we also skip the tracking. This approach is
            good if we use the same socket to send commands to kernel-space
            (that we want to track) and to listen to events (that we do not
            track).
        """
        match = True
        if (seq):
            match = (self._msg_seq == seq)
        return (self._msg_seq and match)

    def portid_ok(self, portid):
        """ Perform portID origin check.

            portid - netlink portid that we want to check

            This method returns true if the origin is fulfilled, otherwise
            false is returned. We skip the tracking for netlink message
            whose portID is zero since it is reserved for event-based
            kernel notifications. On the other hand, if portid is set but
            the message PortID is not (i.e. this is an event message coming
            from kernel-space), then we also skip the tracking. This
            approach is good if we use the same socket to send commands to
            kernel-space (that we want to track) and to listen to events
            (that we do not track).
        """
        match = True
        if (portid):
            match = (self._pid == portid)
        return (self._pid and match)

    def printf_header(self):
        """ This method prints the netlink message header to stdout.
            It may be useful for debugging purposes.  Use Message.printf()
            to print out a full message.

            One example of the output is the following:

            ----------------        ------------------
            |  0000000040  |        | message length |
            | 00016 | R-A- |        |  type | flags  |
            |  1289148991  |        | sequence number|
            |  0000000000  |        |     port ID    |
            ----------------        ------------------

            This example shows the header of the netlink message that
            is sent to kernel-space to set up the link interface eth0.  The
            netlink header data is displayed in base 10.   The possible
            flags in the netlink header are:

                - R, that indicates that NLM_F_REQUEST is set.
                - M, that indicates that NLM_F_MULTI is set.
                - A, that indicates that NLM_F_ACK is set.
                - E, that indicates that NLM_F_ECHO is set.

            The lack of one flag is displayed with '-'.
        """
        request = "-"
        if (self._msg_flags & NLM_F_REQUEST):
            request = "R"
        multi = "-"
        if (self._msg_flags & NLM_F_MULTI):
            multi = "M"
        ack = "-"
        if (self._msg_flags & NLM_F_ACK):
            ack = "A"
        echo = "-"
        if (self._msg_flags & NLM_F_ECHO):
            echo = "E"
        print("----------------\t------------------");
        print("|  %.010u  |\t| message length |" % len(self));
        print("| %.05u | %c%c%c%c |\t|  type | flags  |" %
            (self._msg_type,
            request, multi, ack, echo))
        print("|  %.010u  |\t| sequence number|" % self._msg_seq);
        print("|  %.010u  |\t|     port ID    |" % self._pid);
        print("----------------\t------------------");

    def printf(self, extra_header_size=0):
        """ This method prints the full netlink message to stdout.
            It may be useful for debugging purposes.

            It calls Message.printf_header() and Payload.printf() to do
            the real work.
        """
        self.printf_header()
        self._payload.printf(self._msg_type, extra_header_size)

    def get_binary(self):
        """ Return a packed struct suitable for sending through a
            netlink socket.
        """
        if (not self._payload):
            raise UnboundLocalError("There is no payload in this message")

        self._msg_length = NLMSG_ALIGN(MSG_HDRLEN + len(self._payload))

        return pack(header_format,
                    self._msg_length,
                    self._msg_type,
                    self._msg_flags,
                    self._msg_seq,
                    self._pid) + self._payload.get_binary()

    def get_errno(self):
        """ Return the errno reported by Netlink.
        """
        errno_ = 0
        if (self._msg_type == NLMSG_ERROR):
            # The error code is a signed integer stored in the
            #   first four bytes of the payload
            errno_ = unpack("i", self._payload.get_data()[:4])[0]
            # "Netlink subsystems returns the errno value
            #   with different signess" -- libmnl/src/callback.c
            if (errno_ < 0):
                errno_ = -1 * errno_
        return errno_

    def get_errstr(self):
        """ Return the errno reported by Netlink as
            interpretted by os.strerror.
        """
        return os.strerror(self.get_errno())


class Payload(object):
    def __init__(self, contents=None):
        """ The payload of a netlink message.

            contents - string or object representing the payload
                        When passing an object, it must implement the
                        get_binary() method.
        """
        if (contents):
            self.set(contents)
        else:
            self._contents = b''

    def __len__(self):
        """ Get the length of the payload (in bytes).
        """
        return len(self._contents)

    def __getitem__(self, index):
        """ Return the value at index.
        """
        return self._contents[index]

    def __repr__(self):
        """ Return escaped data string.
        """
        return repr(self._contents)

    def set(self, contents):
        """ Set the payload contents.

            contents - string or object representing the payload
                        When passing an object, it must implement the
                        get_binary() method.
        """
        if (isinstance(contents, str) or isinstance(contents, bytes)):
            # Py2, it's a str; Py3, it's a bytes
            self._contents = contents
        else:
            self._contents = contents.get_binary()
        self._format = repr(NLMSG_ALIGN(len(self._contents))) + "s"

    def format(self):
        """ Get the payload's struct format.
        """
        return self._format

    def printf(self, msg_type, extra_header_size):
        """ This method prints the netlink message payload to stdout.
            It may be useful for debugging purposes.  Use Message.printf()
            to print out a full message.

            One example of the output is the following:

            | 00 00 00 00  |        |  extra header  |
            | 00 00 00 00  |        |  extra header  |
            | 01 00 00 00  |        |  extra header  |
            | 01 00 00 00  |        |  extra header  |
            |00008|--|00003|        |len |flags| type|
            | 65 74 68 30  |        |      data      |       e t h 0
            ----------------        ------------------

            This example shows the payload for the netlink message
            that is send to kernel-space to set up the link interface eth0.
            The extra header and the attribute payloads are expressed in
            base 16.

            The attribute header is shown in base 10. The possible flags in
            the attribute header are:

                - N, that indicates that NLA_F_NESTED is set.
                - B, that indicates that NLA_F_NET_BYTEORDER is set.

            The lack of one flag is displayed with '-'.
        """
        rem = 0
        for index in range(0, len(self), 4):
            # get four bytes of payload for later use
            buf = []
            for char in self._contents[index:index+4]:
                try:
                    # make sure we have a list of numbers, needed in Py2
                    buf.append(ord(char))
                except TypeError:
                    # char is already a number, we're probably in Py3
                    buf.append(char)

            # make a stunted Attr so we can test its attribute-ness later
            one_attr = Attr(packed_data=self._contents[index:index+4])

            if (msg_type < NLMSG_MIN_TYPE):
                # netlink control message
                print("| %.2x %.2x %.2x %.2x  |\t|                |" %
                    (0xff & buf[0],  0xff & buf[1],
                     0xff & buf[2],  0xff & buf[3]))
            elif (extra_header_size > 0):
                # special handling for the extra header
                extra_header_size = extra_header_size - 4
                print("| %.2x %.2x %.2x %.2x  |\t|  extra header  |" %
                    (0xff & buf[0],  0xff & buf[1],
                     0xff & buf[2],  0xff & buf[3]))
            elif ((rem == 0) and (one_attr.get_type() != 0)):
                # this seems like an attribute header
                # Since this looks like an attribute, make a full Attr
                #   with which to work.
                one_attr = Attr(packed_data=self._contents[index:index+one_attr._length])
                line = "|%c[%d;%dm" % (27, 1, 31)
                line = line + "%.5u" % (len(one_attr),)
                line = line + "%c[%dm" % (27, 0)
                line = line + "|"
                line = line + "%c[%d;%dm" % (27, 1, 32)
                nest = "-"
                if (one_attr.is_nested()):
                    nest = "N"
                byteorder = "-"
                if (one_attr.get_type() & pymnl.attributes.NLA_F_NET_BYTEORDER):
                    byteorder = "B"
                line = line + "%c%c" % (nest, byteorder)
                line = line + "%c[%dm" % (27, 0)
                line = line + "|"
                line = line + "%c[%d;%dm" % (27, 1, 34)
                line = line + "%.5u" % (one_attr.get_type(),)
                line = line + "%c[%dm|\t" % (27, 0)
                line = line + "|len |flags| type|"
                print(line)
                if (not one_attr.is_nested()):
                    rem = len(one_attr) - calcsize(pymnl.attributes.header_format)
            elif (rem > 0):
                # this is the attribute payload
                rem = rem - 4;
                line = ("| %.2x %.2x %.2x %.2x  |\t" %
                    (0xff & buf[0],  0xff & buf[1],
                     0xff & buf[2],  0xff & buf[3]))
                line = line + "|      data      |"
                for bindex in range(0, 4):
                    # Convert each value to a string for testing.
                    # Yes, this is the same value we converted into an
                    # integer above.  But that was for Py2 and this is
                    # needed for Py3.  So, we convert and test the 4
                    # buffer bytes for both Py2 and Py3.  This way, the
                    # bitwise ops work above and the isalnum()
                    # test works here.
                    if (not chr(buf[bindex]).isalnum()):
                        buf[bindex] = 0
                line = line + ("\t %c %c %c %c" %
                    (buf[0], buf[1], buf[2], buf[3]))
                print(line)
        print("----------------\t------------------");

    def add_attr(self, attribute):
        """ Add an Attr object to the payload.

            attribute - an Attr object
        """
        self.set(self._contents + attribute.get_binary())

    def get_binary(self):
        """ Return a packed struct suitable for sending through a
            netlink socket.
        """
        # prepare the null padding
        pad = (NLMSG_ALIGN(len(self)) - len(self)) * b'\x00'
        # push the whole package out
        return self._contents + pad

    def get_data(self):
        """ Return the non-header data string.  This is the non-aligned
            version of get_binary().
        """
        return self._contents


class MessageList(list):
    def __init__(self, msg):
        """ Holds the Message objects making up a multipart message.
        """
        if (msg):
            if (isinstance(msg, Message)):
                self.append(msg)
            elif (isinstance(msg, str) or isinstance(msg, bytes)):
                # Py2, it's a str; Py3, it's a bytes
                self.split(msg)
            else:
                raise TypeError("MessageList only accepts Messages " +
                                "or a packed string")

    def size(self):
        """ Return the length (in bytes) of the total MessageList.
        """
        length = 0
        for msg in self:
            length = length + len(msg)
        return length

    def split(self, msg):
        """ Split multipart message into its component messages.
        """
        while (msg):
            one_msg = Message(msg)
            # Is more data than message header calls for available?
            if (len(one_msg) > one_msg._msg_length):
                # make a Message from the right amount of data
                self.append(Message(msg[:one_msg._msg_length]))
                # strip off the data used to make previous Message
                msg = msg[one_msg._msg_length:]
            else:
                self.append(one_msg)
                msg = None










#!/usr/bin/python
# tests/unittests.py -- test interface for pymnl
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
#

from __future__ import print_function

import unittest

import tests.nlsocket
import tests.attributes
import tests.message

# list of 2-tuples with name and test case
test_cases = [ ("Socket", tests.nlsocket.TestSocket),
               ("Attr", tests.attributes.TestAttributes),
               ("Payload", tests.message.TestPayload),
               ("Message", tests.message.TestMessage),
               ("MessageList", tests.message.TestMessageList) ]

for (name, test_case) in (test_cases):
    suite = test_case.suite()
    results = unittest.TestResult()
    suite.run(results)
    print(name, "tests run:", results.testsRun)
    print(name, "errors:", len(results.errors))
    for error in results.errors:
        print(error[0], "-", error[1])
    print(name, "failed tests:", len(results.failures))
    for failure in results.failures:
        print(failure[0], "-", failure[1])




#!/usr/bin/python
#
# genl-family-get.py -- get info about a genl family
# Copyright 2011 Sean Robinson <seankrobinson@gmail.com>
#
# This file is part of the pymnl package, a Python interface
# for netlink sockets.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from __future__ import print_function

from random import randint
import socket
import sys

import pymnl
import pymnl.genl
from pymnl.attributes import Attr
from pymnl.message import Message, Payload
from pymnl.nlsocket import Socket

if (len(sys.argv) != 2):
    print("%s [family name]" % sys.argv[0])
else:
    # init and bind netlink socket
    sock = Socket(pymnl.NETLINK_GENERIC)
    sock.bind(0, 0)

    # init and build request message
    nlmsg = Message()
    nlmsg.set_type(pymnl.genl.GENL_ID_CTRL)
    nlmsg.set_flags(pymnl.message.NLM_F_REQUEST | pymnl.message.NLM_F_ACK)
    nlmsg.set_seq(randint(0, pow(2, 31)))

    # build genl header and add it to the message
    genl_header = pymnl.genl.GenlMessageHeader(
                        command=pymnl.genl.CTRL_CMD_GETFAMILY,
                        version=1)
    nlmsg.put_extra_header(genl_header)

    # init and build a payload
    payload = Payload()

    # add attributes to payload
    family_id = Attr.new_u32(pymnl.genl.CTRL_ATTR_FAMILY_ID,
                                pymnl.genl.GENL_ID_CTRL)
    payload.add_attr(family_id)
    family_name = Attr.new_strz(pymnl.genl.CTRL_ATTR_FAMILY_NAME,
                                    sys.argv[1].encode())
    payload.add_attr(family_name)
    nlmsg.add_payload(payload)

    # send message through socket
    sock.send(nlmsg)

    # read returned message from netlink
    return_msg = sock.recv(flags=socket.MSG_DONTWAIT)

    sock.close()

    # process the messages returned
    for msg in return_msg:
        if (msg.get_errno()):
            # tell the user what error occurred
            print("error:", msg.get_errstr())
        else:
            # setup and parse attributes in message payload
            genl_parser = pymnl.genl.GenlAttrParser()
            attrs = genl_parser.parse(msg.get_payload())
            print("name=%s\tid=%u\tversion=%u\thdrsize=%u\tmaxattr=%u" %
                    (attrs['name'], attrs['id'], attrs['version'],
                    attrs['hdrsize'], attrs['maxattr']))
            print("ops:")
            for op in attrs['ops'].keys():
                print("id-0x%x flags" % (op, ))
            print()
            print("grps:")
            for group in attrs['groups'].keys():
                print("id-0x%x name: %s" % (group, attrs['groups'][group]))

